package gov.faa.ato.esp;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jbpm.api.Execution;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;

public class TestJoinMultiplicity extends JbpmTestCase
  {

  protected void setUp() throws Exception
    {
    super.setUp();
    String deploymentDbid = repositoryService.createDeployment().addResourceFromClasspath( "TestJoinMultiplicity.jpdl.xml" )
        .deploy();
    }

  public void testMe()
    {

    Map< String, Object > variables = new HashMap< String, Object >();
    variables.put( "candidates", "alex, mary, peter" );

    ProcessInstance processInstance = executionService.startProcessInstanceByKey( "TestJoinMultiplicity", variables );

    String processInstanceId = processInstance.getId();
    log.info( "process instance " + processInstanceId + " started" );

    // we should be at Task1

    processInstance = executionService.findProcessInstanceById( processInstanceId );
    Execution mainExection = ( Execution ) processInstance;
    log.info( "the state of the main execution is: " + mainExection.getState() );
    Map< String, Execution > executionsMap = processInstance.getExecutionsMap();
    // child executions should be 0
    log.info( "child executions=" + executionsMap.size() + "  " + executionsMap.toString() );

    // this will fork -- how many executions now
    Task currentTask = taskService.createTaskQuery().processInstanceId( processInstanceId ).uniqueResult();
    taskService.completeTask( currentTask.getId() );

    processInstance = executionService.findProcessInstanceById( processInstanceId );
    mainExection = ( Execution ) processInstance;
    log.info( "the state of the main execution is: " + mainExection.getState() );
    executionsMap = processInstance.getExecutionsMap();
    // child executions should now be 2
    log.info( "child executions=" + executionsMap.size() + "  " + executionsMap.toString() );

    List< Task > list = taskService.createTaskQuery().processInstanceId( processInstanceId ).list();
    log.info( "current tasks:" + list.toString() );

    // complete both of the forked tasks
    for ( Iterator forkedtasks = list.iterator(); forkedtasks.hasNext(); )
      {
      Task forkedtask = ( Task ) forkedtasks.next();
      log.info( "continuing to join 1" );
      taskService.completeTask( forkedtask.getId() );
      }


    processInstance = executionService.findProcessInstanceById( processInstanceId );
    mainExection = ( Execution ) processInstance;
    log.info( "the state of the main execution is: " + mainExection.getState() );
    executionsMap = processInstance.getExecutionsMap();
    // child executions should now be 0?
    log.info( "child executions=" + executionsMap.size() + "  " + executionsMap.toString() );

    
    // we the join has continued to Task-n
    currentTask = taskService.createTaskQuery().processInstanceId( processInstanceId ).uniqueResult();
    taskService.completeTask( currentTask.getId() );
    
    // process is complete?
    assertTrue( executionService.findProcessInstanceById( processInstanceId ) == null );
    
    }

  protected void tearDown() throws Exception
    {
    super.tearDown();
    }

  }
